import { ComponentType } from 'react';
import { IReactionPublic, IReactionOptions, IReactionDisposer } from '@alilc/lowcode-editor-core';
import { ProjectSchema, ComponentMetadata, ComponentAction, NpmInfo, IEditor, CompositeObject, PropsList, NodeSchema, PropsTransducer, IShellModelFactory } from '@alilc/lowcode-types';
import { AssetsJson } from '@alilc/lowcode-utils';
import { Project } from '../project';
import { Node, DocumentModel, ParentalNode, TransformStage } from '../document';
import { ComponentMeta } from '../component-meta';
import { INodeSelector, Component } from '../simulator';
import { Scroller, IScrollable } from './scroller';
import { Dragon, LocateEvent, DragObject } from './dragon';
import { ActiveTracker } from './active-tracker';
import { Detecting } from './detecting';
import { DropLocation, LocationData } from './location';
import { OffsetObserver } from './offset-observer';
import { SettingTopEntry } from './setting';
import { BemToolsManager } from '../builtin-simulator/bem-tools/manager';
export interface DesignerProps {
    editor: IEditor;
    shellModelFactory: IShellModelFactory;
    className?: string;
    style?: object;
    defaultSchema?: ProjectSchema;
    hotkeys?: object;
    simulatorProps?: object | ((document: DocumentModel) => object);
    simulatorComponent?: ComponentType<any>;
    dragGhostComponent?: ComponentType<any>;
    suspensed?: boolean;
    componentMetadatas?: ComponentMetadata[];
    globalComponentActions?: ComponentAction[];
    onMount?: (designer: Designer) => void;
    onDragstart?: (e: LocateEvent) => void;
    onDrag?: (e: LocateEvent) => void;
    onDragend?: (e: {
        dragObject: DragObject;
        copy: boolean;
    }, loc?: DropLocation) => void;
    [key: string]: any;
}
export declare class Designer {
    readonly dragon: Dragon;
    readonly activeTracker: ActiveTracker;
    readonly detecting: Detecting;
    readonly project: Project;
    readonly editor: IEditor;
    readonly bemToolsManager: BemToolsManager;
    readonly shellModelFactory: IShellModelFactory;
    get currentDocument(): DocumentModel;
    get currentHistory(): import("../document").History<NodeSchema>;
    get currentSelection(): import("../document").Selection;
    constructor(props: DesignerProps);
    setupSelection: () => void;
    postEvent(event: string, ...args: any[]): void;
    private _dropLocation?;
    get dropLocation(): DropLocation;
    /**
     * 创建插入位置，考虑放到 dragon 中
     */
    createLocation(locationData: LocationData): DropLocation;
    /**
     * 清除插入位置
     */
    clearLocation(): void;
    createScroller(scrollable: IScrollable): Scroller;
    private oobxList;
    createOffsetObserver(nodeInstance: INodeSelector): OffsetObserver | null;
    private clearOobxList;
    touchOffsetObserver(): void;
    createSettingEntry(nodes: Node[]): SettingTopEntry;
    /**
     * 获得合适的插入位置
     */
    getSuitableInsertion(insertNode?: Node | NodeSchema | NodeSchema[]): {
        target: ParentalNode;
        index?: number;
    } | null;
    private props?;
    setProps(nextProps: DesignerProps): void;
    loadIncrementalAssets(incrementalAssets: AssetsJson): Promise<void>;
    /**
     * 刷新 componentMetasMap，可间接触发模拟器里的 buildComponents
     */
    refreshComponentMetasMap(): void;
    get(key: string): any;
    private _simulatorComponent?;
    get simulatorComponent(): ComponentType<any> | undefined;
    private _simulatorProps?;
    get simulatorProps(): object | ((project: Project) => object);
    private _suspensed;
    get suspensed(): boolean;
    set suspensed(flag: boolean);
    get schema(): ProjectSchema;
    setSchema(schema?: ProjectSchema): void;
    private _componentMetasMap;
    private _lostComponentMetasMap;
    buildComponentMetasMap(metas: ComponentMetadata[]): void;
    createComponentMeta(data: ComponentMetadata): ComponentMeta;
    getGlobalComponentActions(): ComponentAction[] | null;
    getComponentMeta(componentName: string, generateMetadata?: () => ComponentMetadata | null): ComponentMeta;
    getComponentMetasMap(): Map<string, ComponentMeta>;
    get componentsMap(): {
        [key: string]: NpmInfo | Component;
    };
    private propsReducers;
    transformProps(props: CompositeObject | PropsList, node: Node, stage: TransformStage): CompositeObject | PropsList;
    addPropsReducer(reducer: PropsTransducer, stage: TransformStage): void;
    autorun(effect: (reaction: IReactionPublic) => void, options?: IReactionOptions): IReactionDisposer;
    purge(): void;
}
