import { NodeData, NodeSchema, RootSchema, PageSchema, ComponentsMap, DragNodeObject, DragNodeDataObject } from '@alilc/lowcode-types';
import { Project } from '../project';
import { ISimulatorHost } from '../simulator';
import { ComponentMeta } from '../component-meta';
import { DropLocation, Designer } from '../designer';
import { Node, RootNode, ParentalNode } from './node/node';
import { Selection } from './selection';
import { History } from './history';
import { TransformStage, ModalNodesManager } from './node';
export declare type GetDataType<T, NodeType> = T extends undefined ? NodeType extends {
    schema: infer R;
} ? R : any : T;
export declare class DocumentModel {
    /**
     * 根节点 类型有：Page/Component/Block
     */
    rootNode: RootNode | null;
    /**
     * 文档编号
     */
    id: string;
    /**
     * 选区控制
     */
    readonly selection: Selection;
    /**
     * 操作记录控制
     */
    readonly history: History;
    /**
     * 模态节点管理
     */
    readonly modalNodesManager: ModalNodesManager;
    private _nodesMap;
    readonly project: Project;
    readonly designer: Designer;
    private nodes;
    private seqId;
    private emitter;
    private rootNodeVisitorMap;
    /**
     * @deprecated
     */
    private _addons;
    /**
     * 模拟器
     */
    get simulator(): ISimulatorHost | null;
    get nodesMap(): Map<string, Node>;
    get fileName(): string;
    set fileName(fileName: string);
    get focusNode(): any;
    private _drillDownNode;
    drillDown(node: Node | null): void;
    private _modalNode?;
    private _blank?;
    private inited;
    constructor(project: Project, schema?: RootSchema);
    private willPurgeSpace;
    get modalNode(): ParentalNode<NodeSchema>;
    get currentRoot(): any;
    addWillPurge(node: Node): void;
    removeWillPurge(node: Node): void;
    isBlank(): boolean;
    /**
     * 生成唯一id
     */
    nextId(possibleId: string | undefined): string;
    /**
     * 根据 id 获取节点
     */
    getNode(id: string): Node | null;
    /**
     * 根据 id 获取节点
     */
    getNodeCount(): number;
    /**
     * 是否存在节点
     */
    hasNode(id: string): boolean;
    private activeNodes?;
    /**
     * 根据 schema 创建一个节点
     */
    createNode<T extends Node = Node, C = undefined>(data: GetDataType<C, T>, checkId?: boolean): T;
    destroyNode(node: Node): void;
    /**
     * 插入一个节点
     */
    insertNode(parent: ParentalNode, thing: Node | NodeData, at?: number | null, copy?: boolean): Node;
    /**
     * 插入多个节点
     */
    insertNodes(parent: ParentalNode, thing: Node[] | NodeData[], at?: number | null, copy?: boolean): Node<NodeSchema>[];
    /**
     * 移除一个节点
     */
    removeNode(idOrNode: string | Node): void;
    /**
     * 内部方法，请勿调用
     */
    internalRemoveAndPurgeNode(node: Node, useMutator?: boolean): void;
    unlinkNode(node: Node): void;
    private _dropLocation;
    /**
     * 内部方法，请勿调用
     */
    internalSetDropLocation(loc: DropLocation | null): void;
    /**
     * 投放插入位置标记
     */
    get dropLocation(): DropLocation;
    /**
     * 包裹当前选区中的节点
     */
    wrapWith(schema: NodeSchema): Node | null;
    /**
     * 导出 schema 数据
     */
    get schema(): RootSchema;
    import(schema: RootSchema, checkId?: boolean): void;
    export(stage?: TransformStage): PageSchema | import("@alilc/lowcode-types").ComponentSchema;
    /**
     * 导出节点数据
     */
    getNodeSchema(id: string): NodeData | null;
    /**
     * 是否已修改
     */
    isModified(): boolean;
    getComponent(componentName: string): any;
    getComponentMeta(componentName: string): ComponentMeta;
    private _opened;
    private _suspensed;
    /**
     * 是否为非激活状态
     */
    get suspensed(): boolean;
    /**
     * 与 suspensed 相反，是否为激活状态，这个函数可能用的更多一点
     */
    get active(): boolean;
    /**
     * @deprecated 兼容
     */
    get actived(): boolean;
    /**
     * 是否打开
     */
    get opened(): boolean;
    /**
     * 切换激活，只有打开的才能激活
     * 不激活，打开之后切换到另外一个时发生，比如 tab 视图，切换到另外一个标签页
     */
    private setSuspense;
    suspense(): void;
    activate(): void;
    /**
     * 打开，已载入，默认建立时就打开状态，除非手动关闭
     */
    open(): DocumentModel;
    /**
     * 关闭，相当于 sleep，仍然缓存，停止一切响应，如果有发生的变更没被保存，仍然需要去取数据保存
     */
    close(): void;
    /**
     * 从项目中移除
     */
    remove(): void;
    purge(): void;
    checkNesting(dropTarget: ParentalNode, dragObject: DragNodeObject | NodeSchema | Node | DragNodeDataObject): boolean;
    /**
     * @deprecated since version 1.0.16.
     * Will be deleted in version 2.0.0.
     * Use checkNesting method instead.
     */
    checkDropTarget(dropTarget: ParentalNode, dragObject: DragNodeObject | DragNodeDataObject): boolean;
    /**
     * 检查对象对父级的要求，涉及配置 parentWhitelist
     */
    checkNestingUp(parent: ParentalNode, obj: NodeSchema | Node): boolean;
    /**
     * 检查投放位置对子级的要求，涉及配置 childWhitelist
     */
    checkNestingDown(parent: ParentalNode, obj: NodeSchema | Node): boolean;
    getRoot(): RootNode;
    toData(extraComps?: string[]): {
        componentsMap: ComponentsMap;
        utils: any;
        componentsTree: (PageSchema | import("@alilc/lowcode-types").ComponentSchema)[];
    };
    getHistory(): History;
    get root(): RootNode;
    /**
     * @deprecated
     */
    getAddonData(name: string): any;
    /**
     * @deprecated
    */
    exportAddonData(): {};
    /**
     * @deprecated
     */
    registerAddon(name: string, exportData: any): void;
    acceptRootNodeVisitor(visitorName: string, visitorFn: (node: RootNode) => any): {};
    getRootNodeVisitor(name: string): any;
    getComponentsMap(extraComps?: string[]): ComponentsMap;
    /**
     * 获取 schema 中的 utils 节点，当前版本不判断页面中使用了哪些 utils，直接返回资产包中所有的 utils
     * @returns
     */
    getUtilsMap(): any;
    onNodeCreate(func: (node: Node) => void): () => void;
    onNodeDestroy(func: (node: Node) => void): () => void;
    /**
     * @deprecated
     */
    refresh(): void;
    /**
     * @deprecated
     */
    onRefresh(): void;
    onReady(fn: Function): () => void;
    private setupListenActiveNodes;
}
export declare function isDocumentModel(obj: any): obj is DocumentModel;
export declare function isPageSchema(obj: any): obj is PageSchema;
