import Logger from 'zen-logger';
import { CompositeObject, ComponentAction, MetadataTransducer } from '@alilc/lowcode-types';
export declare type PreferenceValueType = string | number | boolean;
export interface ILowCodePluginPreferenceDeclarationProperty {
    key: string;
    description: string;
    type: string;
    default?: PreferenceValueType;
    useMultipleLineTextInput?: boolean;
    enum?: any[];
    enumDescriptions?: string[];
    deprecationMessage?: string;
}
/**
 * declaration of plugin`s preference
 * when strictPluginMode === true， only declared preference can be obtained from inside plugin.
 *
 * @export
 * @interface ILowCodePluginPreferenceDeclaration
 */
export interface ILowCodePluginPreferenceDeclaration {
    title: string;
    properties: ILowCodePluginPreferenceDeclarationProperty[];
}
export declare type PluginPreference = Map<string, Record<string, PreferenceValueType>>;
export interface ILowCodePluginConfig {
    dep?: string | string[];
    init?(): void;
    destroy?(): void;
    exports?(): any;
}
export interface ILowCodePluginConfigMetaEngineConfig {
    lowcodeEngine?: string;
}
export interface ILowCodePluginConfigMeta {
    preferenceDeclaration?: ILowCodePluginPreferenceDeclaration;
    dependencies?: string[];
    engines?: ILowCodePluginConfigMetaEngineConfig;
}
export interface ILowCodePluginCore {
    name: string;
    dep: string[];
    disabled: boolean;
    config: ILowCodePluginConfig;
    logger: Logger;
    on(event: string | symbol, listener: (...args: any[]) => void): any;
    emit(event: string | symbol, ...args: any[]): boolean;
    removeAllListeners(event?: string | symbol): this;
    init(forceInit?: boolean): void;
    isInited(): boolean;
    destroy(): void;
    toProxy(): any;
    setDisabled(flag: boolean): void;
}
interface ILowCodePluginExportsAccessor {
    [propName: string]: any;
}
export declare type ILowCodePlugin = ILowCodePluginCore & ILowCodePluginExportsAccessor;
export interface IDesignerCabin {
    registerMetadataTransducer: (transducer: MetadataTransducer, level: number, id?: string) => void;
    addBuiltinComponentAction: (action: ComponentAction) => void;
    removeBuiltinComponentAction: (actionName: string) => void;
}
export interface IPluginPreferenceMananger {
    getPreferenceValue: (key: string, defaultValue?: PreferenceValueType) => PreferenceValueType | undefined;
}
export interface ILowCodePluginContext {
}
export interface ILowCodePluginContextPrivate {
}
export interface ILowCodePluginContextApiAssembler {
    assembleApis: (context: ILowCodePluginContextPrivate) => void;
}
interface ILowCodePluginManagerPluginAccessor {
    [pluginName: string]: ILowCodePlugin | any;
}
export interface ILowCodePluginManagerCore {
    register(pluginConfigCreator: (ctx: ILowCodePluginContext, pluginOptions?: any) => ILowCodePluginConfig, pluginOptions?: any, options?: CompositeObject): Promise<void>;
    init(pluginPreference?: Map<string, Record<string, PreferenceValueType>>): Promise<void>;
    get(pluginName: string): ILowCodePlugin | undefined;
    getAll(): ILowCodePlugin[];
    has(pluginName: string): boolean;
    delete(pluginName: string): any;
    setDisabled(pluginName: string, flag: boolean): void;
    dispose(): void;
}
export declare type ILowCodePluginManager = ILowCodePluginManagerCore & ILowCodePluginManagerPluginAccessor;
export declare function isLowCodeRegisterOptions(opts: any): opts is ILowCodeRegisterOptions;
export interface ILowCodeRegisterOptions {
    /** Will enable plugin registered with auto-initialization immediately
     * other than plugin-manager init all plugins at certain time.
     * It is helpful when plugin register is later than plugin-manager initialization. */
    autoInit?: boolean;
    /** allow overriding existing plugin with same name when override === true */
    override?: boolean;
}
export interface IPluginContextOptions {
    pluginName: string;
}
export interface IPluginMetaDefinition {
    /** define dependencies which the plugin depends on */
    dependencies?: string[];
    /** specify which engine version is compatible with the plugin */
    engines?: {
        /** e.g. '^1.0.0' */
        lowcodeEngine?: string;
    };
}
interface IPluginConfigCreatorFn<T extends Record<string, any> = Record<string, any>> {
    (ctx: ILowCodePluginContext, pluginOptions?: T): ILowCodePluginConfig;
}
export declare type IPluginConfigCreator<T extends Record<string, any> = Record<string, any>> = IPluginConfigCreatorFn<T> & {
    pluginName: string;
    meta?: IPluginMetaDefinition;
};
export {};
