import { DocumentModel, Node as ComponentNode, ParentalNode } from '../document';
import { LocateEvent } from './dragon';
export interface LocationData {
    target: ParentalNode;
    detail: LocationDetail;
    source: string;
    event: LocateEvent;
}
export declare enum LocationDetailType {
    Children = "Children",
    Prop = "Prop"
}
export interface LocationChildrenDetail {
    type: LocationDetailType.Children;
    index?: number | null;
    /**
     * 是否有效位置
     */
    valid?: boolean;
    edge?: DOMRect;
    near?: {
        node: ComponentNode;
        pos: 'before' | 'after' | 'replace';
        rect?: Rect;
        align?: 'V' | 'H';
    };
    focus?: {
        type: 'slots';
    } | {
        type: 'node';
        node: ParentalNode;
    };
}
export interface LocationPropDetail {
    type: LocationDetailType.Prop;
    name: string;
    domNode?: HTMLElement;
}
export declare type LocationDetail = LocationChildrenDetail | LocationPropDetail | {
    type: string;
    [key: string]: any;
};
export interface Point {
    clientX: number;
    clientY: number;
}
export interface CanvasPoint {
    canvasX: number;
    canvasY: number;
}
export declare type Rects = DOMRect[] & {
    elements: Array<Element | Text>;
};
export declare type Rect = DOMRect & {
    elements: Array<Element | Text>;
    computed?: boolean;
};
export declare function isLocationData(obj: any): obj is LocationData;
export declare function isLocationChildrenDetail(obj: any): obj is LocationChildrenDetail;
export declare function isRowContainer(container: Element | Text, win?: Window): boolean;
export declare function isChildInline(child: Element | Text, win?: Window): boolean;
export declare function getRectTarget(rect: Rect | null): Element | Text;
export declare function isVerticalContainer(rect: Rect | null): boolean;
export declare function isVertical(rect: Rect | null): boolean;
export declare function getWindow(elem: Element | Document): Window;
export declare class DropLocation {
    readonly target: ParentalNode;
    readonly detail: LocationDetail;
    readonly event: LocateEvent;
    readonly source: string;
    get document(): DocumentModel;
    constructor({ target, detail, source, event }: LocationData);
    clone(event: LocateEvent): DropLocation;
    /**
     * @deprecated
     * 兼容 vision
     */
    getContainer(): ParentalNode<import("@alilc/lowcode-types").NodeSchema>;
    /**
     * @deprecated
     * 兼容 vision
     */
    getInsertion(): any;
}
