/// <reference types="node" />
import { IEditor, ISetValueOptions } from '@alilc/lowcode-types';
import { SettingEntry } from './setting-entry';
import { Node } from '../../document';
import { ComponentMeta } from '../../component-meta';
import { Designer } from '../designer';
import { EventEmitter } from 'events';
export declare class SettingPropEntry implements SettingEntry {
    readonly parent: SettingEntry;
    readonly editor: IEditor;
    readonly isSameComponent: boolean;
    readonly isMultiple: boolean;
    readonly isSingle: boolean;
    readonly nodes: Node[];
    readonly componentMeta: ComponentMeta | null;
    readonly designer: Designer;
    readonly top: SettingEntry;
    readonly isGroup: boolean;
    readonly type: 'field' | 'group';
    readonly id: string;
    readonly emitter: EventEmitter;
    private _name;
    get name(): string | number;
    get path(): (string | number)[];
    extraProps: any;
    constructor(parent: SettingEntry, name: string | number, type?: 'field' | 'group');
    getId(): string;
    setKey(key: string | number): void;
    getKey(): string | number;
    remove(): void;
    /**
     * 判断当前属性值是否一致
     * -1 多种值
     * 0 无值
     * 1 类似值，比如数组长度一样
     * 2 单一植
     */
    get valueState(): number;
    /**
     * 获取当前属性值
     */
    getValue(): any;
    /**
     * 设置当前属性值
     */
    setValue(val: any, isHotValue?: boolean, force?: boolean, extraOptions?: ISetValueOptions): void;
    /**
     * 清除已设置的值
     */
    clearValue(): void;
    /**
     * 获取子项
     */
    get(propName: string | number): SettingEntry;
    /**
     * 设置子级属性值
     */
    setPropValue(propName: string | number, value: any): void;
    /**
     * 清除已设置值
     */
    clearPropValue(propName: string | number): void;
    /**
     * 获取子级属性值
     */
    getPropValue(propName: string | number): any;
    /**
     * 获取顶层附属属性值
     */
    getExtraPropValue(propName: string): any;
    /**
     * 设置顶层附属属性值
     */
    setExtraPropValue(propName: string, value: any): void;
    getNode(): Node<import("@alilc/lowcode-types").NodeSchema>;
    getName(): string;
    getProps(): SettingEntry;
    get props(): SettingEntry;
    onValueChange(func: () => any): () => void;
    /**
     * @deprecated
     */
    valueChange(options?: ISetValueOptions): void;
    notifyValueChange(oldValue: any, newValue: any): void;
    getDefaultValue(): any;
    isIgnore(): boolean;
    getVariableValue(): string;
    setVariableValue(value: string): void;
    setUseVariable(flag: boolean): void;
    isUseVariable(): boolean;
    get useVariable(): boolean;
    getMockOrValue(): any;
    internalToShellPropEntry(): import("@alilc/lowcode-types").IPublicModelSettingPropEntry;
}
