"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.Session = exports.History = void 0;
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _events = require("events");
var _lowcodeEditorCore = require("@alilc/lowcode-editor-core");
var History = /*#__PURE__*/function () {
  var _proto = History.prototype;
  _proto.setSerialization = function setSerialization(serialization) {
    this.currentSerialization = serialization;
  };
  function History(dataFn, redoer, timeGap) {
    var _this = this;
    if (timeGap === void 0) {
      timeGap = 1000;
    }
    this.redoer = redoer;
    this.timeGap = timeGap;
    this.session = void 0;
    this.records = void 0;
    this.point = 0;
    this.emitter = new _events.EventEmitter();
    this.asleep = false;
    this.currentSerialization = {
      serialize: function serialize(data) {
        return JSON.stringify(data);
      },
      unserialize: function unserialize(data) {
        return JSON.parse(data);
      }
    };
    this.session = new Session(0, null, this.timeGap);
    this.records = [this.session];
    (0, _lowcodeEditorCore.reaction)(function () {
      return dataFn();
    }, function (data) {
      if (_this.asleep) return;
      (0, _lowcodeEditorCore.untracked)(function () {
        var log = _this.currentSerialization.serialize(data);

        // do not record unchanged data
        if (_this.session.data === log) {
          return;
        }
        if (_this.session.isActive()) {
          _this.session.log(log);
        } else {
          _this.session.end();
          var lastState = _this.getState();
          var _cursor = _this.session.cursor + 1;
          var session = new Session(_cursor, log, _this.timeGap);
          _this.session = session;
          _this.records.splice(_cursor, _this.records.length - _cursor, session);
          var currentState = _this.getState();
          if (currentState !== lastState) {
            _this.emitter.emit('statechange', currentState);
          }
        }
      });
    }, {
      fireImmediately: true
    });
  }
  _proto.isSavePoint = function isSavePoint() {
    return this.point !== this.session.cursor;
  };
  _proto.sleep = function sleep() {
    this.asleep = true;
  };
  _proto.wakeup = function wakeup() {
    this.asleep = false;
  };
  _proto.go = function go(cursor) {
    this.session.end();
    var currentCursor = this.session.cursor;
    cursor = +cursor;
    if (cursor < 0) {
      cursor = 0;
    } else if (cursor >= this.records.length) {
      cursor = this.records.length - 1;
    }
    if (cursor === currentCursor) {
      return;
    }
    var session = this.records[cursor];
    var hotData = session.data;
    this.sleep();
    try {
      this.redoer(this.currentSerialization.unserialize(hotData));
      this.emitter.emit('cursor', hotData);
    } catch (e) /* istanbul ignore next */{
      console.error(e);
    }
    this.wakeup();
    this.session = session;
    this.emitter.emit('statechange', this.getState());
  };
  _proto.back = function back() {
    if (!this.session) {
      return;
    }
    var cursor = this.session.cursor - 1;
    this.go(cursor);
    var editor = _lowcodeEditorCore.globalContext.get(_lowcodeEditorCore.Editor);
    if (!editor) {
      return;
    }
    editor.emit('history.back', cursor);
  };
  _proto.forward = function forward() {
    if (!this.session) {
      return;
    }
    var cursor = this.session.cursor + 1;
    this.go(cursor);
    var editor = _lowcodeEditorCore.globalContext.get(_lowcodeEditorCore.Editor);
    if (!editor) {
      return;
    }
    editor.emit('history.forward', cursor);
  };
  _proto.savePoint = function savePoint() {
    if (!this.session) {
      return;
    }
    this.session.end();
    this.point = this.session.cursor;
    this.emitter.emit('statechange', this.getState());
  }

  /**
   *  |    1     |     1    |    1     |
   *  | -------- | -------- | -------- |
   *  | modified | redoable | undoable |
   */;
  _proto.getState = function getState() {
    var cursor = this.session.cursor;
    var state = 7;
    // undoable ?
    if (cursor <= 0) {
      state -= 1;
    }
    // redoable ?
    if (cursor >= this.records.length - 1) {
      state -= 2;
    }
    // modified ?
    if (this.point === cursor) {
      state -= 4;
    }
    return state;
  };
  _proto.onStateChange = function onStateChange(func) {
    var _this2 = this;
    this.emitter.on('statechange', func);
    return function () {
      _this2.emitter.removeListener('statechange', func);
    };
  };
  _proto.onCursor = function onCursor(func) {
    var _this3 = this;
    this.emitter.on('cursor', func);
    return function () {
      _this3.emitter.removeListener('cursor', func);
    };
  };
  _proto.destroy = function destroy() {
    this.emitter.removeAllListeners();
    this.records = [];
  }
  /**
   *
   * @deprecated
   * @returns
   * @memberof History
   */;
  _proto.isModified = function isModified() {
    return this.isSavePoint();
  };
  (0, _createClass2["default"])(History, [{
    key: "hotData",
    get: function get() {
      return this.session.data;
    }
  }]);
  return History;
}();
exports.History = History;
var Session = /*#__PURE__*/function () {
  function Session(cursor, data, timeGap) {
    if (timeGap === void 0) {
      timeGap = 1000;
    }
    this.cursor = cursor;
    this.timeGap = timeGap;
    this._data = void 0;
    this.activeTimer = void 0;
    this.setTimer();
    this.log(data);
  }
  var _proto2 = Session.prototype;
  _proto2.log = function log(data) {
    if (!this.isActive()) {
      return;
    }
    this._data = data;
    this.setTimer();
  };
  _proto2.isActive = function isActive() {
    return this.activeTimer != null;
  };
  _proto2.end = function end() {
    if (this.isActive()) {
      this.clearTimer();
    }
  };
  _proto2.setTimer = function setTimer() {
    var _this4 = this;
    this.clearTimer();
    this.activeTimer = setTimeout(function () {
      return _this4.end();
    }, this.timeGap);
  };
  _proto2.clearTimer = function clearTimer() {
    if (this.activeTimer) {
      clearTimeout(this.activeTimer);
    }
    this.activeTimer = null;
  };
  (0, _createClass2["default"])(Session, [{
    key: "data",
    get: function get() {
      return this._data;
    }
  }]);
  return Session;
}();
exports.Session = Session;