import { CompositeValue, JSSlot } from '@alilc/lowcode-types';
import { Props } from './props';
import { SlotNode, Node } from '../node';
import { TransformStage } from '../transform-stage';
export declare const UNSET: unique symbol;
export declare type UNSET = typeof UNSET;
export interface IPropParent {
    delete(prop: Prop): void;
    readonly props: Props;
    readonly owner: Node;
    readonly path: string[];
}
export declare type ValueTypes = 'unset' | 'literal' | 'map' | 'list' | 'expression' | 'slot';
export declare class Prop implements IPropParent {
    parent: IPropParent;
    readonly isProp = true;
    readonly owner: Node;
    /**
     * 键值
     */
    key: string | number | undefined;
    /**
     * 扩展值
     */
    spread: boolean;
    readonly props: Props;
    readonly options: any;
    constructor(parent: IPropParent, value?: CompositeValue | UNSET, key?: string | number, spread?: boolean, options?: {});
    setupItems(): Prop[];
    /**
     * @see SettingTarget
     */
    getPropValue(propName: string | number): any;
    /**
     * @see SettingTarget
     */
    setPropValue(propName: string | number, value: any): void;
    /**
     * @see SettingTarget
     */
    clearPropValue(propName: string | number): void;
    readonly id: string;
    private _type;
    /**
     * 属性类型
     */
    get type(): ValueTypes;
    private _value;
    /**
     * 属性值
     */
    get value(): CompositeValue | UNSET;
    export(stage?: TransformStage): CompositeValue;
    private _code;
    /**
     * 获得表达式值
     */
    get code(): string;
    /**
     * 设置表达式值
     */
    set code(code: string);
    getAsString(): string;
    /**
     * set value, val should be JSON Object
     */
    setValue(val: CompositeValue): void;
    getValue(): CompositeValue;
    private dispose;
    private _slotNode?;
    get slotNode(): SlotNode;
    setAsSlot(data: JSSlot): void;
    /**
     * 取消设置值
     */
    unset(): void;
    /**
     * 是否未设置值
     */
    isUnset(): boolean;
    isVirtual(): boolean;
    /**
     * @returns  0: the same 1: maybe & like 2: not the same
     */
    compare(other: Prop | null): number;
    private _items;
    private _maps;
    /**
     * 作为 _maps 的一层缓存机制，主要是复用部分已存在的 Prop，保持响应式关系，比如：
     * 当前 Prop#_value 值为 { a: 1 }，当调用 setValue({ a: 2 }) 时，所有原来的子 Prop 均被销毁，
     * 导致假如外部有 mobx reaction（常见于 observer），此时响应式链路会被打断，
     * 因为 reaction 监听的是原 Prop(a) 的 _value，而不是新 Prop(a) 的 _value。
     */
    private _prevMaps;
    get path(): string[];
    /**
     * 构造 items 属性，同时构造 maps 属性
     */
    private get items();
    private get maps();
    /**
     * 获取某个属性
     * @param createIfNone 当没有的时候，是否创建一个
     */
    get(path: string | number, createIfNone?: boolean): Prop | null;
    /**
     * 从父级移除本身
     */
    remove(): void;
    /**
     * 删除项
     */
    delete(prop: Prop): void;
    /**
     * 删除 key
     */
    deleteKey(key: string): void;
    /**
     * 元素个数
     */
    get size(): number;
    /**
     * 添加值到列表
     *
     * @param force 强制
     */
    add(value: CompositeValue, force?: boolean): Prop | null;
    /**
     * 设置值到字典
     *
     * @param force 强制
     */
    set(key: string | number, value: CompositeValue | Prop, force?: boolean): Prop;
    /**
     * 是否存在 key
     */
    has(key: string): boolean;
    private purged;
    /**
     * 回收销毁
     */
    purge(): void;
    /**
     * 迭代器
     */
    [Symbol.iterator](): {
        next(): {
            value: Prop;
        };
    };
    /**
     * 遍历
     */
    forEach(fn: (item: Prop, key: number | string | undefined) => void): void;
    /**
     * 遍历
     */
    map<T>(fn: (item: Prop, key: number | string | undefined) => T): T[] | null;
    getProps(): Props;
    getNode(): Node<import("@alilc/lowcode-types").NodeSchema>;
}
export declare function isProp(obj: any): obj is Prop;
export declare function isValidArrayIndex(key: any, limit?: number): key is number;
