import { Designer } from '../designer';
import { DocumentModel } from '../document';
import { ProjectSchema, RootSchema, TransformStage } from '@alilc/lowcode-types';
import { ISimulatorHost } from '../simulator';
export declare class Project {
    readonly designer: Designer;
    private emitter;
    readonly documents: DocumentModel[];
    private data;
    private _simulator?;
    /**
     * 模拟器
     */
    get simulator(): ISimulatorHost | null;
    constructor(designer: Designer, schema?: ProjectSchema);
    get currentDocument(): DocumentModel;
    private _config;
    get config(): any;
    set config(value: any);
    private _i18n;
    get i18n(): any;
    set i18n(value: any);
    private getComponentsMap;
    /**
     * 获取项目整体 schema
     */
    getSchema(stage?: TransformStage): ProjectSchema;
    /**
     * 替换当前document的schema,并触发渲染器的render
     * @param schema
     */
    setSchema(schema?: ProjectSchema): void;
    /**
     * 整体设置项目 schema
     *
     * @param autoOpen true 自动打开文档 string 指定打开的文件
     */
    load(schema?: ProjectSchema, autoOpen?: boolean | string): void;
    /**
     * 卸载当前项目数据
     */
    unload(): void;
    removeDocument(doc: DocumentModel): void;
    /**
     * 分字段设置储存数据，不记录操作记录
     */
    set(key: 'version' | 'componentsTree' | 'componentsMap' | 'utils' | 'constants' | 'i18n' | 'css' | 'dataSource' | string, value: any): void;
    /**
     * 分字段设置储存数据
     */
    get(key: 'version' | 'componentsTree' | 'componentsMap' | 'utils' | 'constants' | 'i18n' | 'css' | 'dataSource' | 'config' | string): any;
    private documentsMap;
    getDocument(id: string): DocumentModel | null;
    getDocumentByFileName(fileName: string): DocumentModel | null;
    createDocument(data?: RootSchema): DocumentModel;
    open(doc?: string | DocumentModel | RootSchema): DocumentModel | null;
    checkExclusive(activeDoc: DocumentModel): void;
    closeOthers(opened: DocumentModel): void;
    /**
     * 提供给模拟器的参数
     */
    get simulatorProps(): object;
    private mountSimulator;
    setRendererReady(renderer: any): void;
    onSimulatorReady(fn: (args: any) => void): () => void;
    onRendererReady(fn: (args: any) => void): () => void;
    onCurrentDocumentChange(fn: (doc: DocumentModel) => void): () => void;
}
